# This file is auto-generated by the Perl DateTime Suite time zone
# code generator (0.07) This code generator comes with the
# DateTime::TimeZone module distribution in the tools/ directory
#
# Generated from ../../data/tz/Olson/northamerica.  Olson data version 2003e
#
# Do not edit this file directly.
#
package DateTime::TimeZone::America::Martinique;

use strict;

use Class::Singleton;
use DateTime::TimeZone;
use DateTime::TimeZone::OlsonDB;

@DateTime::TimeZone::America::Martinique::ISA = ( 'Class::Singleton', 'DateTime::TimeZone' );

my $spans =
[
    [
DateTime::TimeZone::NEG_INFINITY,
59611176260,
DateTime::TimeZone::NEG_INFINITY,
59611161600,
-14660,
0,
'LMT'
    ],
    [
59611176260,
60284145860,
59611161600,
60284131200,
-14660,
0,
'FFMT'
    ],
    [
60284145860,
62459524800,
60284131460,
62459510400,
-14400,
0,
'AST'
    ],
    [
62459524800,
62474641200,
62459514000,
62474630400,
-10800,
1,
'ADT'
    ],
    [
62474641200,
DateTime::TimeZone::INFINITY,
62474655600,
DateTime::TimeZone::INFINITY,
-14400,
0,
'AST'
    ],
];

sub _max_year { 2013 }

sub _new_instance
{
    return shift->_init( @_, spans => $spans );
}



1;

